@@ -21,7 +21,6 @@ import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.IdentityHashMap;
@@ -29,7 +28,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import net.morilib.automata.AlphabetsNotDiscreteException;
 import net.morilib.automata.DFA;
 import net.morilib.automata.DFAState;
 import net.morilib.automata.NFA;
@@ -42,6 +40,7 @@ import net.morilib.automata.nfa.RegexParseException;
 import net.morilib.automata.nfa.RegexpParser;
 import net.morilib.range.CharSets;
 import net.morilib.range.Interval;
+import net.morilib.range.IntervalsInt;
 import net.morilib.range.Range;
 import net.morilib.range.RangeAdder;
 import net.morilib.util.TreeSectionMap;
@@ -88,14 +87,38 @@ public class DFABuilder implements NinaAction {
 
 	private static class SQ {
 
+		private Range r;
 		private int num;
 
-		SQ(int n) {
+		SQ(Range r, int n) {
+			this.r = r;
 			num = n;
 		}
 
 	}
 
+	private static class DY {
+
+		private String var;
+
+		DY(String s) {
+			var = s;
+		}
+
+	}
+
+	private static class RR {
+
+		private String name;
+		private Object delim;
+
+		RR(String s, Object d) {
+			name = s;
+			delim = d;
+		}
+
+	}
+
 	/**
 	 * 
 	 *
@@ -117,6 +140,8 @@ public class DFABuilder implements NinaAction {
 		DBS others = null;
 		DBS end = null;
 		DBS recursive = null;
+		String dynamicVariable;
+		DBS dynamic = null;
 
 		PrintWriter out = null;
 		int devoirFermer = 0;
@@ -132,13 +157,6 @@ public class DFABuilder implements NinaAction {
 				mealy = new TreeSectionMap<Interval, Object, Object>(
 						r, m);
 			} else {
-				for(Tuple2<Interval, DBS> w : edges.entries()) {
-					if(!o.independentOf(w.getA()) &&
-							!d.equals(w.getB())) {
-						throw new NinaNondeterministicException(
-								o + "," + d + "," + w);
-					}
-				}
 				edges.insert(r, d);
 				mealy.insert(r, m);
 			}
@@ -205,6 +223,11 @@ public class DFABuilder implements NinaAction {
 			userMealy.put(l, m);
 		}
 
+		void linkDynamic(DBS d, String var) {
+			dynamicVariable = var;
+			dynamic = d;
+		}
+
 		void sortir(Object o) {
 			String s;
 
@@ -317,6 +340,22 @@ public class DFABuilder implements NinaAction {
 			return userMealy != null ? userMealy.get(s) : null;
 		}
 
+		/**
+		 * 
+		 * @return
+		 */
+		public String getDynamicVariable() {
+			return dynamicVariable;
+		}
+
+		/**
+		 * 
+		 * @return
+		 */
+		public DBS getDynamic() {
+			return dynamic;
+		}
+
 		@Override
 		public DFAState<Object, Object, Void> go(Object a) {
 			DFAState<Object, Object, Void> d;
@@ -475,237 +514,6 @@ public class DFABuilder implements NinaAction {
 
 	};
 
-	private static class FA
-	implements DFA<Object, Object, Void>, NFA<Object, Object, Void> {
-
-		private String name;
-		private DBS initial;
-
-		private transient Set<Object> allStates;
-
-		FA(String name, DBS init) {
-			this.name = name;
-			initial   = init;
-			allStates = new HashSet<Object>(DFAs.allStateRanges(init));
-		}
-
-		@Override
-		public boolean isState(NFAState o) {
-			return allStates.contains(o);
-		}
-
-		@Override
-		public Set<NFAState> getStates(NFAState s, Object a) {
-			DFAState<Object, Object, Void> d;
-
-			if(!isState(s)) {
-				return Collections.emptySet();
-			} else if((d = ((DBS)s).go(a)).isDead()) {
-				return Collections.emptySet();
-			} else {
-				return Collections.singleton((NFAState)d);
-			}
-		}
-
-		@Override
-		public Set<NFAState> getStates(NFAState s, Range r) {
-			Set<NFAState> x;
-			DBS d;
-
-			if(!isState(s)) {
-				return Collections.emptySet();
-			} else {
-				d = (DBS)s;
-				x = new HashSet<NFAState>();
-				for(Tuple2<Interval, DBS> t : d.edges.entries()) {
-					for(Interval v : r.intervals()) {
-						if(t.getA().equals(v)) {
-							x.add(t.getB());
-						}
-					}
-				}
-				return x;
-			}
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getStates(net.morilib.automata.NFAState, java.util.EnumSet)
-		 */
-		@Override
-		public Set<NFAState> getStates(NFAState s,
-				EnumSet<TextBound> b) {
-			return Collections.emptySet();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getStatesEpsilon(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public Set<NFAState> getStatesEpsilon(NFAState s) {
-			return Collections.emptySet();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getStatesBound(net.morilib.automata.NFAState, java.util.EnumSet)
-		 */
-		@Override
-		public Set<NFAState> getStatesBound(NFAState s,
-				EnumSet<TextBound> b) {
-			return Collections.emptySet();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getInitialStates()
-		 */
-		@Override
-		public Set<NFAState> getInitialStates() {
-			return Collections.singleton((NFAState)initial);
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#isInitialState(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public boolean isInitialState(NFAState o) {
-			return initial.equals(o);
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#isFinal(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public boolean isFinal(NFAState s) {
-			return isState(s) && ((DBS)s).isAccepted();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#isFinalAny(java.util.Set)
-		 */
-		@Override
-		public boolean isFinalAny(Set<NFAState> ss) {
-			for(NFAState s : ss) {
-				if(isFinal(s))  return true;
-			}
-			return false;
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getEdges(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public NFAEdges<Object> getEdges(NFAState s) {
-			return isState(s) ? (DBS)s : null;
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#nextAlphabets(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public Set<Interval> nextAlphabets(NFAState s) {
-			Set<Interval> x = new HashSet<Interval>();
-			DBS d;
-
-			if(isState(s)) {
-				d = (DBS)s;
-				for(Tuple2<Interval, DBS> t : d.edges.entries()) {
-					x.add(t.getA());
-				}
-				return x;
-			} else {
-				return Collections.emptySet();
-			}
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#nextAlphabets(java.util.Set)
-		 */
-		@Override
-		public Iterable<Interval> nextAlphabets(Set<NFAState> ss) {
-			Set<Interval> x = new HashSet<Interval>();
-
-			for(NFAState s : ss) {
-				x.addAll(nextAlphabets(s));
-			}
-			return x;
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#nextDiscreteAlphabets(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public Set<Object> nextDiscreteAlphabets(NFAState s) {
-			throw new AlphabetsNotDiscreteException();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#nextDiscreteAlphabets(java.util.Set)
-		 */
-		@Override
-		public Iterable<Object> nextDiscreteAlphabets(
-				Set<NFAState> s) {
-			throw new AlphabetsNotDiscreteException();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getAcceptedStates()
-		 */
-		@Override
-		public Set<NFAState> getAcceptedStates() {
-			Set<NFAState> x = new HashSet<NFAState>();
-
-			for(Object s : allStates) {
-				if(((DBS)s).isAccepted())  x.add((DBS)s);
-			}
-			return x;
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getMatchTag(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public Set<Void> getMatchTag(NFAState s) {
-			return Collections.emptySet();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getMatchTagEnd(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public Set<Void> getMatchTagEnd(NFAState s) {
-			return Collections.emptySet();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#getAccept(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public Set<Object> getAccept(NFAState s) {
-			if(isAccepted(s)) {
-				return Collections.singleton((Object)s);
-			} else {
-				return Collections.emptySet();
-			}
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.NFA#isAccepted(net.morilib.automata.NFAState)
-		 */
-		@Override
-		public boolean isAccepted(NFAState s) {
-			return isState(s) && ((DBS)s).isAccepted();
-		}
-
-		/* (non-Javadoc)
-		 * @see net.morilib.automata.DFA#getInitialState()
-		 */
-		@Override
-		public DFAState<Object, Object, Void> getInitialState() {
-			initial.sortirMoore();
-			return initial;
-		}
-
-	}
-
 	//
 	private static final Object _END = new Object();
 
@@ -715,8 +523,10 @@ public class DFABuilder implements NinaAction {
 			new IdentityHashMap<Object, String>();
 	private Map<String, String> typemap =
 			new HashMap<String, String>();
+	private Map<String, String> dynmap =
+			new HashMap<String, String>();
 	private DBS initial, vertex;
-	private FA accepted = null;
+	private BuildMachine accepted = null;
 	private int mealyType;
 	private Object mealy;
 	private String name;
@@ -778,19 +588,17 @@ public class DFABuilder implements NinaAction {
 		return _linkv(q, x, vertex, r, o);
 	}
 
-	private void _linkdfa(NinaEvent q, DFA<Object, NFAState, Void> d,
-			DBS v, Object o) {
+	private void _linkdfa1(NinaEvent q, DFA<Object, NFAState, Void> d,
+			DBS v, DBS z, Object o, String lh, String lc, boolean en) {
 		Collection<DFAState<Object, NFAState, Void>> c;
 		Map<DFAState<Object, NFAState, Void>, DBS> m;
 		DFAState<Object, NFAState, Void> h;
-		String g, lh, lc;
+		String g;
 		DBS x, w;
 
-		lh = q.getLanguageOption().getLookahead();
-		lc = q.getLanguageOption().getLookaheadCommit();
-		c  = DFAs.allStateRanges(d.getInitialState());
-		m  = new HashMap<DFAState<Object, NFAState, Void>, DBS>();
-		g  = lh + lc;
+		c = DFAs.allStateRanges(d.getInitialState());
+		m = new HashMap<DFAState<Object, NFAState, Void>, DBS>();
+		g = lh + lc;
 		if(o != null)  g += o.toString();
 		for(DFAState<Object, NFAState, Void> y : c) {
 			if(y.equals(d.getInitialState())) {
@@ -799,7 +607,11 @@ public class DFABuilder implements NinaAction {
 				m.put(y, x = new DBS());
 			}
 
-			if(y.isAccepted())  x.linkOthers(vertex, g);
+			if(y.isAccepted()) {
+				x.linkOthers(z, g);
+				if(en)  x.linkEnd(z, o);
+			}
+
 			for(Interval a : y.getAlphabetRanges()) {
 				h = y.go(a.getInfimumBound());
 				if(h.equals(d.getInitialState())) {
@@ -814,10 +626,22 @@ public class DFABuilder implements NinaAction {
 		}
 	}
 
-	private void _linkdq(NinaEvent q, CharSequence s, DBS v) {
-		List<DoubleQuoteTokenizer.Elems> b;
+	private void _linkre(NinaEvent q, String re, DBS x, String lh,
+			String lc) {
 		NFA<Object, NFAState, Void> n;
 		DFA<Object, NFAState, Void> d;
+
+		try {
+			n = RegexpParser.parse(re);
+			d = ConvertedRangeDFA.convertDFA(n);
+			_linkdfa1(q, d, x, vertex, mealy, lh, lc, false);
+		} catch (RegexParseException e) {
+			throw new NinaException("invalidregex");
+		}
+	}
+
+	private void _linkdq(NinaEvent q, CharSequence s, DBS v) {
+		List<DoubleQuoteTokenizer.Elems> b;
 		String g, lh, lm, lc;
 		boolean z;
 		int k, c;
@@ -844,13 +668,9 @@ public class DFABuilder implements NinaAction {
 
 			if(p.getRegex() != null) {
 				x = _linkc(q, x, s.charAt(k), lh + lm);
-				try {
-					n = RegexpParser.parse(p.getRegex());
-					d = ConvertedRangeDFA.convertDFA(n);
-					_linkdfa(q, d, x, mealy);
-				} catch (RegexParseException e) {
-					throw new NinaException("invalidregex");
-				}
+				_linkre(q, p.getRegex(), x,
+						q.getLanguageOption().getLookahead(),
+						q.getLanguageOption().getLookaheadCommit());
 			} else if(p.getLookahead() != null) {
 				x = _linkc(q, x, s.charAt(k), lh);
 				g = lm + lh;
@@ -898,7 +718,7 @@ public class DFABuilder implements NinaAction {
 		}
 	}
 
-	private void _linksq(NinaEvent q, DBS v, int len) {
+	private void _linksq(NinaEvent q, DBS v, Range r, int len) {
 		NinaLanguageOption p = q.getLanguageOption();
 		String s, t = "", m;
 		DBS x = v, y;
@@ -912,7 +732,12 @@ public class DFABuilder implements NinaAction {
 			} else if(mealyType == '#') {
 				t = p.getAssignArray(m, k);
 			}
-			x.linkOthers(y, s + t);
+
+			if(r == null) {
+				x.linkOthers(y, s + t);
+			} else {
+				x.linkAlphabet(y, r, s + t);
+			}
 			s = "";
 		}
 
@@ -923,7 +748,62 @@ public class DFABuilder implements NinaAction {
 		} else {
 			t = m;
 		}
-		x.linkOthers(vertex, t);
+
+		if(r == null) {
+			x.linkOthers(vertex, t);
+		} else {
+			x.linkAlphabet(vertex, r, t);
+		}
+	}
+
+	@SuppressWarnings("unchecked")
+	private void _linkrepr(NinaEvent q, DBS w, String sub, Object dl) {
+		NinaLanguageOption l = q.getLanguageOption();
+		DBS x, y, z, v;
+		String t, s;
+		char[] a;
+		int k;
+
+		s = l.getUnget();
+		t = mealy != null ? s + mealy.toString() : s;
+
+		v = new DBS();
+		w.linkOthers(v, l.getUnget() + l.getInitList());
+
+		if(dl == null || dl.equals("")) {
+			v.linkRecursive(vertex, sub);
+			vertex.linkOthers(v, s);
+		} else if(dl instanceof String) {
+			x = new DBS();
+			v.linkRecursive(x, sub);
+			x.label = l.getAddList();
+			x.linkAlphabet(v, dl, null);
+			x.linkOthers(vertex, t);
+			x.linkEnd(vertex, mealy);
+		} else if(dl instanceof DFA) {
+			x = new DBS();
+			v.linkRecursive(x, sub);
+			x.label = l.getAddList();
+			_linkdfa1(q, (DFA<Object, NFAState, Void>)dl, x, v, mealy,
+					"", s, true);
+			x.linkOthers(vertex, t);
+			x.linkEnd(vertex, mealy);
+		} else if((a = (char[])dl).length < 1) {
+			v.linkRecursive(vertex, sub);
+			vertex.linkOthers(v, s);
+		} else {
+			x = z = new DBS();
+			v.linkRecursive(z, sub);
+			z.label = l.getAddList();
+			for(k = 0; k < a.length - 1; k++) {
+				x.linkAlphabet(y = new DBS(),
+						IntervalsInt.newPoint(a[k]), null);
+				x = y;
+			}
+			x.linkAlphabet(v, IntervalsInt.newPoint(a[k]), null);
+			z.linkOthers(vertex, t);
+			z.linkEnd(vertex, mealy);
+		}
 	}
 
 	/* (non-Javadoc)
@@ -937,20 +817,24 @@ public class DFABuilder implements NinaAction {
 		vertex = (ver != null) ? (DBS)ver : createState(q);
 		if((o = q.getEdge()) == null) {
 			v.linkOthers(vertex, q.getLanguageOption().getUnget());
-		} else if(o instanceof FA) {
-			v.linkRecursive(vertex, ((FA)o).name);
+		} else if(o instanceof BuildMachine) {
+			v.linkRecursive(vertex, ((BuildMachine)o).name);
 		} else if(o instanceof RT) {
 			v.linkRecursive(vertex, ((RT)o).name);
 		} else if(o instanceof W) {
 			_link1(q, v);
 		} else if(o instanceof SQ) {
-			_linksq(q, v, ((SQ)o).num);
+			_linksq(q, v, ((SQ)o).r, ((SQ)o).num);
 		} else if(o.equals(CharSets.ALL_CHAR)) {
 			v.linkOthers(vertex, mealy);
 		} else if(o instanceof Range) {
 			v.linkAlphabet(vertex, (Range)o, mealy);
 		} else if(o instanceof UE) {
 			v.linkUser(vertex, ((UE)o).edge, mealy);
+		} else if(o instanceof DY) {
+			v.linkDynamic(vertex, ((DY)o).var);
+		} else if(o instanceof RR) {
+			_linkrepr(q, v, ((RR)o).name, ((RR)o).delim);
 		} else if(o == _END) {
 			v.linkEnd(vertex, mealy);
 		} else {
@@ -1020,15 +904,15 @@ public class DFABuilder implements NinaAction {
 			NinaParser p,
 			String s,
 			Map<String, String> map,
-			Map<String, Object> sub) {
-		String t = Nina.getFAName(s);
+			NinaSubautomata sub) {
+		String t = Nina.getFAName(sub, s);
 
 		if(sub.containsKey(t)) {
 			q.setEdge(new RT(t));
 		} else {
 			sub.put(t, null);
 			q.setEdge(p.compileSubautomaton(s, new DFABuilder(), map,
-					sub).getMachine());
+					sub, null).getMachine());
 		}
 	}
 
@@ -1049,6 +933,14 @@ public class DFABuilder implements NinaAction {
 		q.setEdge(_END);
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeGrammar(net.morilib.nina.NinaEvent, java.lang.String)
+	 */
+	@Override
+	public void setEdgeGrammar(NinaEvent q, String name) {
+		q.setEdge(new RT("g:" + name));
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.NinaAction#accept()
 	 */
@@ -1056,7 +948,7 @@ public class DFABuilder implements NinaAction {
 	public NinaAction accept(String name) {
 		if(accepted == null) {
 			this.name = name;
-			accepted = new FA(name, initial);
+			accepted = new BuildMachine(name, initial);
 		}
 		return this;
 	}
@@ -1176,8 +1068,8 @@ public class DFABuilder implements NinaAction {
 	 * @see net.morilib.nina.NinaAction#setEdgeSequence(net.morilib.nina.NinaEvent)
 	 */
 	@Override
-	public void setEdgeSequence(NinaEvent q, int len) {
-		q.setEdge(new SQ(len));
+	public void setEdgeSequence(NinaEvent q, Range r, int len) {
+		q.setEdge(new SQ(r, len));
 	}
 
 	/* (non-Javadoc)
@@ -1188,4 +1080,44 @@ public class DFABuilder implements NinaAction {
 		mealyType = c;
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setAccept(boolean)
+	 */
+	@Override
+	public void setAccept(boolean accept) {
+		vertex.accepted = accept;
+	}
+
+	@Override
+	public void setEdgeDynamic(NinaEvent q, String var) {
+		q.setEdge(new DY(var));
+		dynmap.put(var, var);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#getDynamicTransitMap()
+	 */
+	@Override
+	public Map<String, String> getDynamicTransitMap() {
+		return Collections.unmodifiableMap(dynmap);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeRepeatedResource(net.morilib.nina.NinaEvent, java.lang.String, java.lang.Object)
+	 */
+	@Override
+	public void setEdgeRepeatedResource(NinaEvent q, String s,
+			Object d) {
+		q.setEdge(new RR(s, d));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#empty(net.morilib.nina.NinaEvent, net.morilib.nina.NinaConsole)
+	 */
+	@Override
+	public NinaAction empty(NinaEvent q, String name) {
+		setStartState(createState(q));
+		return accept(name);
+	}
+
 }
