@@ -15,11 +15,19 @@
  */
 package net.morilib.nina;
 
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.regex.Matcher;
@@ -38,11 +46,12 @@ public final class Nina {
 	/**
 	 * 
 	 */
-	public static final String VERSION = "0.3.1";
+	public static final String VERSION = "0.4.5";
 
 	//
 	private static final Pattern FANM = Pattern.compile(
 			"(.*/)?([^/.]+)(\\..*)?");
+	private static final Pattern CDLM = Pattern.compile("-+");
 
 	//
 	private Nina() {}
@@ -130,15 +139,171 @@ public final class Nina {
 		}
 	}
 
+	static String getAutomatonName(String s) {
+		if(s.equalsIgnoreCase("full")) {
+			return "DFABuilder";
+		} else if(s.equalsIgnoreCase("NFA")) {
+			return "NFABuilder";
+		} else if(s.equalsIgnoreCase("Markov")) {
+			return "MarkovBuilder";
+		} else if(s.endsWith("Builder")) {
+			return s;
+		} else {
+			throw new NinaException("subautomatonnotfound", s);
+		}
+	}
+
+	//
+	static int len(char c) {
+		Character.UnicodeBlock b;
+
+		b = Character.UnicodeBlock.of(c);
+		if(b == null) {
+			return 1;
+		} else if(c >= 0xff01 && c <= 0xff60) {
+			return 2;
+		} else if(c >= 0xffe0 && c <= 0xffe6) {
+			return 2;
+		} else if(b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY) ||
+				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_FORMS) ||
+				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS) ||
+				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT) ||
+				b.equals(Character.UnicodeBlock.CJK_RADICALS_SUPPLEMENT) ||
+//				b.equals(Character.UnicodeBlock.CJK_STROKES) ||
+				b.equals(Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION) ||
+				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS) ||
+				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A) ||
+				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B) ||
+				b.equals(Character.UnicodeBlock.KATAKANA) ||
+				b.equals(Character.UnicodeBlock.HIRAGANA) ||
+				b.equals(Character.UnicodeBlock.HANGUL_SYLLABLES)) {
+			return 2;
+		} else {
+			return 1;
+		}
+	}
+
+	static int len(String s) {
+		int r = 0;
+
+		for(int i = 0; i < s.length(); i++) {
+			r = r + len(s.charAt(i));
+		}
+		return r;
+	}
+
+	static int[] toiarray(String s) {
+		int[] a;
+		int m;
+
+		a = new int[len(s)];
+		for(int i = 0, j = 0; j < s.length(); i += m, j++) {
+			m = len(s.charAt(j));
+			a[i] = s.charAt(j);
+			for(int k = i + 1; k < i + m; k++) {
+				a[k] = Quadro.EQ_TO_LEFT;
+			}
+		}
+		return a;
+	}
+
+	//
+	static InputStream findLib(String n, List<String> libs) {
+		File f;
+
+		if((f = new File(n)).isFile()) {
+			try {
+				return new FileInputStream(f);
+			} catch(FileNotFoundException e) {
+				throw new RuntimeException(e);
+			}
+		} else {
+			for(String s : libs) {
+				f = new File(new File(s), n);
+				if(f.isFile()) {
+					try {
+						return new FileInputStream(f);
+					} catch(FileNotFoundException e) {
+						throw new RuntimeException(e);
+					}
+				}
+			}
+			return null;
+		}
+	}
+
+	private static final Pattern PTN1 = Pattern.compile(
+			"#header[ \t]+(.*)[ \t]+(.*)([ \t]+(.*))?");
+
+	static String[] readHeader(String fn,
+			List<String> l) throws IOException {
+		String[] a = new String[3];
+		BufferedReader r = null;
+		InputStream i;
+		Matcher m;
+		String s;
+
+		try {
+			if((i = findLib(fn, l)) == null) {
+				throw new NinaException("filenotfound", fn);
+			}
+			r = new BufferedReader(new InputStreamReader(i));
+			s = r.readLine();
+			if(s != null && (m = PTN1.matcher(s)).matches()) {
+				a[0] = m.group(1);
+				a[1] = m.group(2);
+				a[2] = m.group(4);
+			}
+			return a;
+		} finally {
+			if(r != null)  r.close();
+		}
+	}
+
+	static List<int[]> readImport(String fn, List<String> lb,
+			String enc) throws IOException {
+		BufferedReader r = null;
+		boolean b = false;
+		List<int[]> l;
+		Reader d;
+		String s;
+
+		try {
+			if(enc != null) {
+				try {
+					d = new InputStreamReader(findLib(fn, lb), enc);
+				} catch(UnsupportedEncodingException e) {
+					d = new InputStreamReader(findLib(fn, lb));
+				}
+			} else {
+				d = new InputStreamReader(findLib(fn, lb));
+			}
+			r = new BufferedReader(d);
+			l = new ArrayList<int[]>();
+			while((s = r.readLine()) != null) {
+				if(b) {
+					b = !CDLM.matcher(s).matches();
+				} else if(!(b = CDLM.matcher(s).matches())){
+					l.add(toiarray(s));
+				}
+			}
+			return l;
+		} finally {
+			if(r != null)  r.close();
+		}
+	}
+
 	/**
 	 * 
 	 * @param s
 	 * @return
 	 */
-	public static String getFAName(String s) {
+	public static String getFAName(NinaSubautomata b, String s) {
 		Matcher m;
 
-		if(!(m = FANM.matcher(s)).matches()) {
+		if(b != null && NinaSubautomata.MAIN.equals(s)) {
+			return b.getMainname();
+		} else if(!(m = FANM.matcher(s)).matches()) {
 			throw new RuntimeException();
 		} else {
 			return m.group(2);
@@ -153,10 +318,10 @@ public final class Nina {
 	 * @return
 	 */
 	public static NinaAction compile(String name, String s,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons) {
-		return NinaParser.compile(Quadro.read(name, s), null, null,
-				prms, sub, cons, 0);
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons) {
+		return NinaParser.compile(Quadro.read(name, s, l), null, null,
+				prms, sub, cons, cons, 0);
 	}
 
 	/**
@@ -171,10 +336,10 @@ public final class Nina {
 	 */
 	public static NinaAction compile(String name, Reader rd,
 			List<String> libs, Map<String, String> prms,
-			Map<String, Object> sub,
+			NinaSubautomata sub,
 			NinaConsole cons) throws IOException {
-		return NinaParser.compile(Quadro.read(name, rd), null, libs,
-				prms, sub, cons, 0);
+		return NinaParser.compile(Quadro.read(name, rd, libs), null,
+				libs, prms, sub, cons, cons, 0);
 	}
 
 	/**
@@ -187,11 +352,10 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaAction compile(String name, Reader rd,
-			Map<String, String> prms,
-			Map<String, Object> sub,
-			NinaConsole cons) throws IOException {
-		return NinaParser.compile(Quadro.read(name, rd), null, null,
-				prms, sub, cons, 0);
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons) throws IOException {
+		return NinaParser.compile(Quadro.read(name, rd, l), null, null,
+				prms, sub, cons, cons, 0);
 	}
 
 	/**
@@ -203,11 +367,10 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaAction compileResource(String name,
-			Map<String, String> prms,
-			Map<String, Object> sub,
-			NinaConsole cons) throws IOException {
-		return NinaParser.compile(Quadro.readResource(name),
-				null, null, prms, sub, cons, 0);
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons) throws IOException {
+		return NinaParser.compile(Quadro.readResource(name, l),
+				null, null, prms, sub, cons, cons, 0);
 	}
 
 	/**
@@ -219,10 +382,10 @@ public final class Nina {
 	 * @return
 	 */
 	public static NinaAction trace(String name, String s,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons, boolean pause) {
-		return NinaParser.compile(Quadro.read(name, s), null, null,
-				prms, sub, cons,
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons, boolean pause) {
+		return NinaParser.compile(Quadro.read(name, s, l), null, null,
+				prms, sub, cons, cons,
 				NinaParser._PRINT | (pause ? NinaParser._PAUSE : 0));
 	}
 
@@ -237,10 +400,11 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaAction trace(String name, Reader rd,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons, boolean pause) throws IOException {
-		return NinaParser.compile(Quadro.read(name, rd), null, null,
-				prms, sub, cons,
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons,
+			boolean pause) throws IOException {
+		return NinaParser.compile(Quadro.read(name, rd, l), null, null,
+				prms, sub, cons, cons,
 				NinaParser._PRINT | (pause ? NinaParser._PAUSE : 0));
 	}
 
@@ -254,10 +418,11 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaAction traceResource(String name,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons, boolean pause) throws IOException {
-		return NinaParser.compile(Quadro.readResource(name),
-				null, null, prms, sub, cons,
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons,
+			boolean pause) throws IOException {
+		return NinaParser.compile(Quadro.readResource(name, l),
+				null, null, prms, sub, cons, cons,
 				NinaParser._PRINT | (pause ? NinaParser._PAUSE : 0));
 	}
 
@@ -282,9 +447,24 @@ public final class Nina {
 	 * @return
 	 */
 	public static NinaPattern pattern(String name, String s,
-			Map<String, String> prms, Map<String, Object> sub,
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons) {
+		return cst(compile(name, s, prms, sub, l, cons).getMachine());
+	}
+
+	/**
+	 * 
+	 * @param name
+	 * @param s
+	 * @param prms
+	 * @param sub
+	 * @return
+	 */
+	public static NinaPattern pattern(String name, String s,
+			Map<String, String> prms, NinaSubautomata sub,
 			NinaConsole cons) {
-		return cst(compile(name, s, prms, sub, cons).getMachine());
+		return cst(compile(name, s, prms, sub,
+				Collections.<String>emptyList(), cons).getMachine());
 	}
 
 	/**
@@ -297,9 +477,26 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaPattern pattern(String name, Reader rd,
-			Map<String, String> prms, Map<String, Object> sub,
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons) throws IOException {
+		return cst(compile(name, rd, prms, sub, l, cons));
+	}
+
+	/**
+	 * 
+	 * @param name
+	 * @param rd
+	 * @param prms
+	 * @param sub
+	 * @param cons
+	 * @return
+	 * @throws IOException
+	 */
+	public static NinaPattern pattern(String name, Reader rd,
+			Map<String, String> prms, NinaSubautomata sub,
 			NinaConsole cons) throws IOException {
-		return cst(compile(name, rd, prms, sub, cons));
+		return cst(compile(name, rd, prms, sub,
+				Collections.<String>emptyList(), cons));
 	}
 
 	/**
@@ -311,9 +508,10 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaPattern patternResource(String name,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons) throws IOException {
-		return cst(compileResource(name, prms, sub, cons).getMachine());
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons) throws IOException {
+		return cst(compileResource(
+				name, prms, sub, l, cons).getMachine());
 	}
 
 	/**
@@ -325,9 +523,9 @@ public final class Nina {
 	 * @return
 	 */
 	public static NinaPattern patternTrace(String name, String s,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons, boolean z) {
-		return cst(trace(name, s, prms, sub, cons, z));
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons, boolean z) {
+		return cst(trace(name, s, prms, sub, l, cons, z));
 	}
 
 	/**
@@ -341,9 +539,10 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaPattern patternTrace(String name, Reader rd,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons, boolean z) throws IOException {
-		return cst(trace(name, rd, prms, sub, cons, z));
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons,
+			boolean z) throws IOException {
+		return cst(trace(name, rd, prms, sub, l, cons, z));
 	}
 
 	/**
@@ -356,9 +555,10 @@ public final class Nina {
 	 * @throws IOException
 	 */
 	public static NinaPattern patternResourceTrace(String name,
-			Map<String, String> prms, Map<String, Object> sub,
-			NinaConsole cons, boolean z) throws IOException {
-		return cst(traceResource(name, prms, sub, cons, z));
+			Map<String, String> prms, NinaSubautomata sub,
+			List<String> l, NinaConsole cons,
+			boolean z) throws IOException {
+		return cst(traceResource(name, prms, sub, l, cons, z));
 	}
 
 }
