@@ -15,9 +15,6 @@
  */
 package net.morilib.nina;
 
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
@@ -27,18 +24,21 @@ import java.io.StringWriter;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import net.morilib.automata.DFA;
 import net.morilib.automata.NFA;
 import net.morilib.automata.NFAState;
+import net.morilib.automata.dfa.ConvertedRangeDFA;
 import net.morilib.automata.nfa.RegexParseException;
 import net.morilib.automata.nfa.RegexpParser;
 import net.morilib.nina.lib.NinaLibrary;
 import net.morilib.range.CharSets;
-import net.morilib.range.CharsetParser;
 
 public class NinaParser {
 
@@ -56,11 +56,14 @@ public class NinaParser {
 
 	//
 	private static enum S {
-		INIT,  PRAG,  PACK,  MACHN, TYPE,
-		OPTN,  OPTV,  OPTQ,  ALIAS, ALI_V, ALI_Q, IMPR, COMNT,
+		INIT,  PRAG0, PRAG,  LICN1,
+		PACK,  MACHN, TYPE,  TYP2,  TYP3,
+		OPTN,  OPTV,  OPTQ,  ALIAS, ALI_V, ALI_Q,
+		PLB_1, PLB_2, PDF_1, PDF_2, PDF_3, REQ1,  REQ2, VTYP1,
+		COMNT,
 		DEF1,  DEF2,  DEF3,  DEF4,  DEFI1, DEFI2,
-		LINIT, LINI2, FI_L1, FI_L2, FI_L3, FI_L4,
-		FI_E,  FI_E2, FI_S,  FI_W,  FI_N,
+		LINIT, LINI2, FI_L0, FI_L1, FI_L2, FI_L3, FI_L4,
+		FI_E,  FI_E2, FI_S,  FI_S2, FI_W,  FI_W2, FI_N,
 		LQUOT, LCSET, LGNFA, LSNFA,
 		PRI01, PRI02, PRI03, PRI04,
 		LPRI1, LPRI2,
@@ -71,11 +74,11 @@ public class NinaParser {
 		FRSE,  FRSE2, FR_W,  FRSW,  FRSW2, FR_N,
 		AR_N,  AR_E,  AREN0, AREN1, AR_EQ, AR_Eq, ARED1, ARED2,
 		AR_ER, ARER1, ARER2, ARER3, ARER4,
-		AR_EI, AR_Ei, AR_EG, AR_Eg, AR_ES, AR_EM, AR_EX,
+		AR_EI, AR_Ei, AR_EG, AR_Eg, AR_ES, AR_EM, AR_EX, AREE1, AREE2,
 		AR_S,  AR_W,  ARWN0, ARWN1, ARWN2, AR_WQ,
 		AR_Wq, AR_Wy, AR_Wn, AR_Wr, AR_Wt, AR_W0, ARWD1, ARWD2,
-		AR_WR, ARWR1, ARWR2, ARWR3, ARWR4,
-		AR_WI, AR_Wi, AR_WG, AR_Wg, AR_WS, AR_WM, AR_WX,
+		AR_WR, ARWR1, ARWR2, ARWR3, ARWR4, ARWR5,
+		AR_WI, AR_Wi, ARWi2, AR_WG, AR_Wg, AR_WS, AR_WM, AR_WX,
 		AR_ZN, AR_ZS, AR_ZE, AR_ZW,
 		F2_W,  F2_N,  F3_W,  F3_N,
 		BR_N,  BR_E,  BR_S,  BR_W,
@@ -92,12 +95,29 @@ public class NinaParser {
 		SER1X, SER1Y,
 		WP_E2, WP_W2,
 		TOIX2, TOIY2,
-		SER2E, SER2F, SER2G, SER2H,
-		SER2W, SER2X, SER2Y,
+		SER2E, SER2F, SER2G, SER2H, SER2W, SER2X, SER2Y,
+		ONS_0, ONS_N, ONS_E, ONS_S, ONS_W,
+		ONS_n, ONS_e, ONS_s, ONS_w
 	}
 
 	//
 	private static final Pattern ANYS = Pattern.compile("\\.([0-9]+)");
+	private static final Pattern CHSS = Pattern.compile(
+			"\\[(.*)\\]([0-9]+)");
+	private static final Pattern REPS = Pattern.compile(
+			"([^\\+]*)\\+(.*)");
+	private static final Pattern LB_Q = Pattern.compile("'(.*)'");
+	private static final Pattern LB_D = Pattern.compile("\"(.*)\"");
+	private static final Pattern LB_S = Pattern.compile("\\[(.*)\\]");
+	private static final Pattern LB_R = Pattern.compile("`(.*)`");
+	private static final Pattern LB_L = Pattern.compile("\\$\\{(.*)\\}");
+	private static final Pattern LD_H = Pattern.compile(
+			"0x([0-9A-Fa-f]{1,4})");
+	private static final Pattern LD_O = Pattern.compile(
+			"01?[0-7]{1,5}");
+	private static final Pattern LD_D = Pattern.compile("[0-9]+");
+	private static final String NL = System.getProperty(
+			"line.separator");
 
 	//
 	static final String STARTSYM = "S";
@@ -115,7 +135,7 @@ public class NinaParser {
 
 	//
 	Map<String, String> params = new HashMap<String, String>();
-	Map<String, Object> subfas = new HashMap<String, Object>();
+	NinaSubautomata subfas;
 	private List<String> libs = Collections.emptyList();
 	private StringBuffer buf = new StringBuffer();
 	private StringBuffer bf2 = new StringBuffer();
@@ -123,49 +143,27 @@ public class NinaParser {
 	private PrintWriter  def;
 	private Class<?> type = Character.TYPE;
 	private NinaAction action;
-	private NinaConsole console;
+	private NinaConsole console, infoput;
 	private List<String> lst;
 	private boolean enddead = false;
 	private int nestlv;
 	private int pause;
 	private int quote;
 	private int chr;
+	private int counter;
 
 	//
 	NinaParser(Quadro q, NinaAction action,
 			Map<String, String> prms,
-			Map<String, Object> subs,
-			NinaConsole cons) {
+			NinaSubautomata subs,
+			NinaConsole cons,
+			NinaConsole info) {
 		this.q = q;
 		this.action  = action;
 		this.params  = new HashMap<String, String>(prms);
 		this.subfas  = subs;
 		this.console = cons;
-	}
-
-	//
-	private InputStream findLib(String n) {
-		File f;
-
-		if((f = new File(n)).isFile()) {
-			try {
-				return new FileInputStream(f);
-			} catch(FileNotFoundException e) {
-				throw new RuntimeException(e);
-			}
-		} else {
-			for(String s : libs) {
-				f = new File(new File(s), n);
-				if(f.isFile()) {
-					try {
-						return new FileInputStream(f);
-					} catch(FileNotFoundException e) {
-						throw new RuntimeException(e);
-					}
-				}
-			}
-			return null;
-		}
+		this.infoput = info;
 	}
 
 	/**
@@ -176,10 +174,9 @@ public class NinaParser {
 	 * @param subs
 	 * @return
 	 */
-	public NinaAction compileSubautomaton(String n,
-			NinaAction action,
-			Map<String, String> prms,
-			Map<String, Object> subs) {
+	public NinaAction compileSubautomaton(String n, NinaAction action,
+			Map<String, String> prms, NinaSubautomata subs,
+			String qtype) {
 		InputStream in = null;
 		NinaAction r;
 		Quadro q;
@@ -187,18 +184,19 @@ public class NinaParser {
 
 		try {
 			console.print("process", n);
-			s = Nina.getFAName(n);
+			s = Nina.getFAName(subfas, n);
 			if((q = this.q.getSubQuadro(n)) != null) {
 				q = q.prototype(this.q);
-			} else if((in = findLib(n)) != null) {
-				q = Quadro.read(s, new InputStreamReader(in));
+			} else if((in = Nina.findLib(n, libs)) != null) {
+				q = Quadro.read(s, new InputStreamReader(in), libs);
 			} else if((in = NinaLibrary.loadLibrary(n)) != null) {
-				q = Quadro.read(s, new InputStreamReader(in));
+				q = Quadro.read(s, new InputStreamReader(in), libs);
 			} else {
-				throw new NinaException("subautomatonnotfound",
-						n);
+				throw new NinaException("subautomatonnotfound", n);
 			}
-			r = compile(q, action, libs, prms, subs, console, pause);
+			if(qtype != null)  q.type = qtype;
+			r = compile(q, action, libs, prms, subs, console, infoput,
+					pause);
 			console.print("processed", n);
 			return r;
 		} catch(IOException e) {
@@ -219,12 +217,15 @@ public class NinaParser {
 			NinaAction action,
 			List<String> libs,
 			Map<String, String> prms,
-			Map<String, Object> subs,
+			NinaSubautomata subs,
 			NinaConsole cons,
+			NinaConsole info,
 			int p) {
-		NinaParser c = new NinaParser(q, action, prms, subs, cons);
-		NinaAction o;
+		NinaAction o, a;
+		NinaParser c;
+		String x;
 
+		c = new NinaParser(q, action, prms, subs, cons, info);
 		try {
 			if(libs != null)  c.libs = libs;
 			while((o = c.step()) == null) {
@@ -235,6 +236,44 @@ public class NinaParser {
 					c.printTrace(System.out);
 				}
 			}
+
+			if(action == null && c.action instanceof DFABuilder) {
+				for(Map.Entry<String, LRObject> t : q.getLRObjects().entrySet()) {
+					LRObject l = t.getValue();
+					int z;
+
+					if((z = l.howManyShiftReduceConflicts()) > 0) {
+						info.pwarn("lrshiftreduceconflict", z,
+								t.getKey());
+					}
+
+					if((z = l.howManyReduceReduceConflicts()) > 0) {
+						info.pwarn("lrreducereduceconflict", z,
+								t.getKey());
+					}
+
+					for(String s : l.getSubautomata().keySet()) {
+						a = c.compileSubautomaton(s, new DFABuilder(),
+								c.params, c.subfas, null);
+						c.subfas.put(s, a);
+					}
+
+					x = l.getDefinition().getLexerName();
+					if(x != null && x.length() > 0) {
+						a = c.compileSubautomaton(x, new NFABuilder(),
+								c.params, c.subfas, null);
+						c.subfas.put(x, a);
+					}
+				}
+
+				if(q.lexer != null && !c.subfas.containsKey(q.lexer)) {
+					a = c.compileSubautomaton(q.lexer,
+							new NFABuilder(), c.params, c.subfas,
+							q.ltype);
+					c.subfas.put(q.lexer, a);
+				}
+				q.checkRequire();
+			}
 		} catch(IOException e) {
 			throw new NinaException(e, "ioerror");
 		}
@@ -252,14 +291,16 @@ public class NinaParser {
 	public static NinaAction complie(Quadro q,
 			List<String> libs,
 			Map<String, String> prms,
-			Map<String, Object> subs,
-			NinaConsole cons) {
+			NinaSubautomata subs,
+			NinaConsole cons,
+			NinaConsole info) {
 		return NinaParser.compile(q,
 				null,
 				libs != null ? new ArrayList<String>(libs) : null,
 				prms,
 				subs,
 				cons,
+				info,
 				0);
 	}
 
@@ -272,9 +313,11 @@ public class NinaParser {
 	 */
 	public static NinaAction complie(Quadro q,
 			Map<String, String> prms,
-			Map<String, Object> subs,
-			NinaConsole cons) {
-		return NinaParser.compile(q, null, null, prms, subs, cons, 0);
+			NinaSubautomata subs,
+			NinaConsole cons,
+			NinaConsole info) {
+		return NinaParser.compile(q, null, null, prms, subs, cons,
+				info, 0);
 	}
 
 	private String getstr(char c) {
@@ -346,23 +389,69 @@ public class NinaParser {
 	}
 
 	private void metBordRes(String s, Map<String, String> mp) {
+		NFA<Object, NFAState, Void> n;
+		DFA<Object, NFAState, Void> d;
+		String t, v;
 		Matcher m;
 
 		if(s.startsWith("ex:")) {
 			action.setUserEdge(q, s.substring(3));
+		} else if(s.startsWith("dy:")) {
+			action.setEdgeDynamic(q, s.substring(3));
+		} else if(s.startsWith("g:")) {
+			action.setEdgeGrammar(q, s.substring(2));
 		} else if(s.indexOf("$c") >= 0) {
 			action.setUserEdge(q, s);
 		} else if((m = ANYS.matcher(s)).matches()) {
-			action.setEdgeSequence(q, Integer.valueOf(m.group(1)));
+			action.setEdgeSequence(q, null,
+					Integer.valueOf(m.group(1)));
+		} else if((m = CHSS.matcher(s)).matches()) {
+			action.setEdgeSequence(q, CharSets.parse(m.group(1)),
+					Integer.valueOf(m.group(2)));
+		} else if((m = REPS.matcher(s)).matches()) {
+			if(!subfas.containsKey(v = m.group(1))) {
+				subfas.put(v, null);
+				compileSubautomaton(v, action, mp, subfas, null);
+			}
+
+			if((t = m.group(2)) == null) {
+				action.setEdgeRepeatedResource(q, v, null);
+			} else if(!type.equals(Character.TYPE)) {
+				action.setEdgeRepeatedResource(q, v, t);
+			} else if(t.length() < 2) {
+				action.setEdgeRepeatedResource(q, v, t.toCharArray());
+			} else if(t.charAt(0) == '"' &&
+					t.charAt(t.length() - 1) == '"') {
+				action.setEdgeRepeatedResource(q, v,
+						t.substring(1, t.length() - 1).toCharArray());
+			} else if(t.charAt(0) == '`' &&
+					t.charAt(t.length() - 1) == '`') {
+				try {
+					t = t.substring(1, t.length() - 1);
+					n = RegexpParser.parse(t);
+					d = ConvertedRangeDFA.convertDFA(n);
+					action.setEdgeRepeatedResource(q, v, d);
+				} catch (RegexParseException e) {
+					throw new NinaException("invalidregex");
+				}
+			} else {
+				action.setEdgeRepeatedResource(q, v, t.toCharArray());
+			}
 		} else {
-			action.setEdgeResource(q, this, s, mp, subfas);
+			t = Nina.getFAName(subfas, s);
+			if(action instanceof DFABuilder &&
+					!subfas.containsKey(t)) {
+				subfas.put(t, null);
+				compileSubautomaton(t, action, mp, subfas, null);
+			}
+			action.setEdgeResource(q, this, t, mp, subfas);
 		}
 	}
 
 	private void metBordCharset(String s) {
 		if(type.equals(Character.TYPE)) {
-//			action.setEdge(q, CharSets.parse(s, q.getCharset()));
-			action.setEdge(q, CharsetParser.parse(s, q.getCharset()));
+			action.setEdge(q, CharSets.parse(s, q.getCharset()));
+//			action.setEdge(q, CharsetParser.parse(s, q.getCharset()));
 		} else {
 			throw new ClassCastException();
 		}
@@ -372,6 +461,82 @@ public class NinaParser {
 		action.setEdge(q, s);
 	}
 
+	private void metBordLabel(String l) {
+		NFA<Object, NFAState, Void> nz;
+		Set<String> d;
+		Matcher m;
+		String s;
+
+		if(q.defines.containsKey(l)) {
+			metBordDefine(l);
+			return;
+		}
+
+		d = new HashSet<String>();
+		for(String x = l;;) {
+			if(d.contains(x)) {
+				throw new NinaException("labelcirculate");
+			}
+			d.add(x);
+			s = q.labelalias.get(x);
+
+			if((m = LB_Q.matcher(s)).matches()) {
+				metBord(m.group(1), '\'');
+				return;
+			} else if((m = LB_D.matcher(s)).matches()) {
+				metBord(m.group(1), '"');
+				return;
+			} else if((m = LB_S.matcher(s)).matches()) {
+				metBordCharset(m.group(1));
+				return;
+			} else if((m = LB_R.matcher(s)).matches()) {
+				try {
+					nz = RegexpParser.parse(m.group(1));
+					metBordNFA(nz);
+					return;
+				} catch(RegexParseException e) {
+					throw new NinaException(this, "invalidregex");
+				}
+			} else if((m = LB_L.matcher(s)).matches()) {
+				x = m.group(1);
+			} else {
+				metBord(s, '\'');
+				return;
+			}
+		}
+	}
+
+	private void metBordDefine(String l) {
+		Set<String> d = new HashSet<String>();
+		Matcher m;
+		String s;
+		int a;
+
+		for(String x = l;;) {
+			if(d.contains(x)) {
+				throw new NinaException("labelcirculate");
+			}
+			d.add(x);
+			s = q.defines.get(x);
+
+			if((m = LD_H.matcher(s)).matches()) {
+				metBord((char)(a = Integer.valueOf(m.group(1), 16)));
+				q.consts.put(l, a);
+				return;
+			} else if((m = LD_O.matcher(s)).matches()) {
+				metBord((char)(a = Integer.valueOf(s, 8)));
+				q.consts.put(l, a);
+				return;
+			} else if((m = LD_D.matcher(s)).matches()) {
+				metBord((char)(a = Integer.valueOf(s)));
+				q.consts.put(l, a);
+				return;
+			} else {
+				x = s;
+			}
+		}
+	}
+
 	private void metmealy() {
 		action.setMealyEdgeType(chr);
 		action.setMealyEdge(buf.reverse().toString());
@@ -382,6 +547,41 @@ public class NinaParser {
 		return params.containsKey(s) ? params.get(s) : "";
 	}
 
+	private NinaAction getAutomatonClass(String s) {
+		String t;
+
+		t = Nina.getAutomatonName(s);
+		pragmas.put("machine", t);
+		q.options.put("machine", t);
+		if(t.indexOf('.') < 0) {
+			t = getClass().getPackage().getName() + "." + t;
+		}
+		return (NinaAction)Nina.prendClasse(t);
+	}
+
+	private void settype(String s) {
+		if(q.type != null) {
+			return;
+		} else if(s.equals("char")) {
+			type = Character.TYPE;
+		} else if(s.equals("int")) {
+			type = Integer.TYPE;
+		} else if(s.equals("long")) {
+			type = Long.TYPE;
+		} else if(s.equals("float")) {
+			type = Float.TYPE;
+		} else if(s.equals("double")) {
+			type = Double.TYPE;
+		} else if(s.equals("string")) {
+			type = String.class;
+		} else if(s.equals("class")) {
+			type = Class.class;
+		} else {
+			type = (Class<?>)Nina.prendClasse(s);
+		}
+		pragmas.put("type", q.type = s);
+	}
+
 	//
 	NinaAction step() {
 		NFA<Object, NFAState, Void> nz;
@@ -392,30 +592,63 @@ public class NinaParser {
 		String s, t;
 		Object o;
 		int c;
+		S z;
 
-		switch(etat) {
+		switch(z = etat) {
 		case INIT:
 			if(q.get() == '#') {
 				buf = new StringBuffer();
 				q.east();
-				etat = S.PRAG;
+				etat = S.PRAG0;
 			} else if(q.get() == '%') {
 				q.east();
 				etat = S.DEF1;
 			} else if(q.isBlankY()) {
-				throw new NinaException(this, "unexpectedeof");
+				// an empty file
+//				throw new NinaException(this, "unexpectedeof");
+				infoput.pwarn("emptylanguage");
+				n = new DFABuilder();
+				t = q.getName();
+				if(subfas.containsKey(t))  subfas.put(t, n);
+				return n.empty(q, t);
 			} else if(!q.isBlankX()) {
 				etat = S.LINIT;
 			}
 			break;
+		case PRAG0:
+			if(q.get() == ' ') {
+				q.east();
+				etat = S.LICN1;
+			} else if(q.isBlankX()) {
+				q.license.append(NL);
+				q.cr();
+				etat = S.INIT;
+			} else {
+				etat = S.PRAG;
+			}
+			break;
+		case LICN1:
+			if(q.isBlankX()) {
+				q.license.append(NL);
+				q.cr();
+				etat = S.INIT;
+			} else if(q.isBlankY()) {
+				throw new NinaException(this, "unexpectedeof");
+			} else if(q.get() >= 0) {
+				q.license.append((char)q.get());
+				q.east();
+			} else {
+				q.east();
+			}
+			break;
 		case PRAG:
 			if(q.get() == Quadro.EQ_TO_LEFT) {
 				q.east();
 			} else if(q.isLetter()) {
 				buf.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else if(!Character.isWhitespace(q.get())) {
-				q.east();
+				q.set(' ');  q.east();
 				etat = S.COMNT;
 			} else if(buf.toString().equals("package")) {
 				buf = new StringBuffer();
@@ -433,24 +666,36 @@ public class NinaParser {
 				buf = new StringBuffer();
 				q.east();
 				etat = S.OPTN;
-			} else if(buf.toString().equals("import")) {
+			} else if(buf.toString().equals("label")) {
 				buf = new StringBuffer();
 				q.east();
-				etat = S.IMPR;
+				etat = S.PLB_1;
+			} else if(buf.toString().equals("define")) {
+				buf = new StringBuffer();
+				q.east();
+				etat = S.PDF_1;
 			} else if(buf.toString().equals("alias")) {
 				buf = new StringBuffer();
 				q.east();
 				etat = S.ALIAS;
+			} else if(buf.toString().equals("require")) {
+				buf = new StringBuffer();
+				q.east();
+				etat = S.REQ1;
+			} else if(buf.toString().equals("valtype")) {
+				buf = new StringBuffer();
+				q.east();
+				etat = S.VTYP1;
 			} else {
 				etat = S.COMNT;
 			}
 			break;
 		case PACK:
 			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
+				q.set(' ');  q.east();
 			} else if(q.isLetter() || q.isDigit() || q.get() == '.') {
 				buf.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				q.setRootPackage(s = buf.toString().trim());
 				q.setRootResource("/" + s.replace('.', '/'));
@@ -460,20 +705,15 @@ public class NinaParser {
 			break;
 		case MACHN:
 			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
+				q.set(' ');  q.east();
 			} else if(q.isLetter() || q.get() == '.') {
 				buf.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				s = buf.toString().trim();
-				pragmas.put("machine", s);
-				q.options.put("machine", s);
-				if(s.indexOf('.') < 0) {
-					s = getClass().getPackage().getName() + "." + s;
-				}
+				n = getAutomatonClass(s);
+				t = Nina.getFAName(subfas, q.getName());
 
-				n = (NinaAction)Nina.prendClasse(s);
-				t = Nina.getFAName(q.getName());
 				if(subfas.containsKey(t))  subfas.put(t, n);
 				if(action == null) {
 					action = n;
@@ -483,6 +723,9 @@ public class NinaParser {
 				} else if(action.getClass().equals(NFABuilder.class) &&
 						n.getClass().equals(DFABuilder.class)) {
 					// do nothing
+				} else if(action.getClass().equals(NFABuilder.class) &&
+						n.getClass().equals(NFABuilder.class)) {
+					// do nothing
 				} else if(!action.getClass().equals(n.getClass())) {
 					throw new NinaException(this, "unknownmachine", s);
 				}
@@ -490,49 +733,56 @@ public class NinaParser {
 			}
 			break;
 		case TYPE:
-			s = null;
-			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
+			if(q.type != null) {
+				etat = S.COMNT;
+			} else if(q.get() == Quadro.EQ_TO_LEFT) {
+				q.set(' ');  q.east();
+			} else if(q.get() == '>') {
+				q.set(' ');  q.east();
+				bf2 = new StringBuffer();
+				etat = S.TYP2;
 			} else if(q.isLetter() || q.get() == '.') {
 				buf.appendCodePoint(q.get());
-				q.east();
-			} else if((s = buf.toString().trim()).equals("char")) {
-				type = Character.TYPE;
-				etat = S.COMNT;
-			} else if(s.equals("int")) {
-				type = Integer.TYPE;
-				etat = S.COMNT;
-			} else if(s.equals("long")) {
-				type = Long.TYPE;
-				etat = S.COMNT;
-			} else if(s.equals("float")) {
-				type = Float.TYPE;
-				etat = S.COMNT;
-			} else if(s.equals("double")) {
-				type = Double.TYPE;
-				etat = S.COMNT;
-			} else if(s.equals("string")) {
-				type = String.class;
-				etat = S.COMNT;
-			} else if(s.equals("class")) {
-				type = Class.class;
-				etat = S.COMNT;
+				q.set(' ');  q.east();
 			} else {
-				type = (Class<?>)Nina.prendClasse(s);
+				settype(buf.toString().trim());
+				etat = S.COMNT;
+			}
+			break;
+		case TYP2:
+			if(q.get() == '>') {
+				q.lexer = bf2.toString().trim();
+				bf2 = new StringBuffer();
+				q.set(' ');  q.east();
+				etat = S.TYP3;
+			} else if(q.isLetter() || q.get() == '.') {
+				bf2.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				settype(buf.toString().trim());
+				etat = S.COMNT;
+			}
+			break;
+		case TYP3:
+			if(q.isLetter() || q.get() == '.') {
+				bf2.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				q.ltype = buf.toString().trim();
+				settype(bf2.toString().trim());
 				etat = S.COMNT;
 			}
-			if(s != null)  pragmas.put("type", q.type = s);
 			break;
 		case OPTN:
 			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
+				q.set(' ');  q.east();
 			} else if(q.get() == '=') {
 				bf2 = new StringBuffer();
-				q.east();
+				q.set(' ');  q.east();
 				etat = S.OPTV;
 			} else if(q.get() >= 0) {
 				buf.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				q.options.put(buf.toString(), "");
 				etat = S.COMNT;
@@ -540,47 +790,40 @@ public class NinaParser {
 			break;
 		case OPTV:
 			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
+				q.set(' ');  q.east();
 			} else if(q.get() == '\'') {
-				q.east();
+				q.set(' ');  q.east();
 				etat = S.OPTQ;
 			} else if(q.get() >= 0) {
 				bf2.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				s = buf.toString();
-				if(s.equals("default") && !subfas.containsKey(
-						t = Nina.getFAName(bf2.toString()))) {
-					subfas.put(t, null);
-					compileSubautomaton(t, new DFABuilder(),
-							Collections.<String, String>emptyMap(),
-							subfas);
-				}
 				q.options.put(s, bf2.toString());
 				etat = S.COMNT;
 			}
 			break;
 		case OPTQ:
 			if(q.get() == '\'') {
-				q.east();
+				q.set(' ');  q.east();
 				etat = S.OPTV;
 			} else if(q.get() >= 0) {
 				bf2.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				throw new NinaException(this, "invalidoption");
 			}
 			break;
 		case ALIAS:
 			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
+				q.set(' ');  q.east();
 			} else if(q.get() == '=') {
 				bf2 = new StringBuffer();
-				q.east();
+				q.set(' ');  q.east();
 				etat = S.ALI_V;
 			} else if(q.get() >= 0) {
 				buf.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				q.options.put(buf.toString(), "");
 				etat = S.COMNT;
@@ -588,13 +831,13 @@ public class NinaParser {
 			break;
 		case ALI_V:
 			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
+				q.set(' ');  q.east();
 			} else if(q.get() == '\'') {
-				q.east();
+				q.set(' ');  q.east();
 				etat = S.ALI_Q;
 			} else if(q.get() >= 0) {
 				bf2.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				q.options.put(buf.toString(), bf2.toString());
 				etat = S.COMNT;
@@ -602,24 +845,118 @@ public class NinaParser {
 			break;
 		case ALI_Q:
 			if(q.get() == '\'') {
-				q.east();
+				q.set(' ');  q.east();
 				etat = S.ALI_V;
 			} else if(q.get() >= 0) {
 				bf2.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
 				throw new NinaException(this, "invalidalias");
 			}
 			break;
-		case IMPR:
-			if(q.get() == Quadro.EQ_TO_LEFT) {
-				q.east();
-			} else if(q.isLetter() || q.isDigit() || q.get() == '.') {
+		case PLB_1:
+			if(q.get() == Quadro.EQ_TO_LEFT || q.get() == '\r') {
+				q.set(' ');  q.east();
+			} else if(q.get() == '=') {
+				bf2 = new StringBuffer();
+				q.set(' ');  q.east();
+				etat = S.PLB_2;
+			} else if(q.get() >= 0) {
 				buf.appendCodePoint(q.get());
-				q.east();
+				q.set(' ');  q.east();
 			} else {
-				s = buf.toString().trim();
-				q.addImports(s);
+				throw new NinaException(this, "invalidlabeldef");
+			}
+			break;
+		case PLB_2:
+			if(q.get() == Quadro.EQ_TO_LEFT || q.get() == '\r') {
+				q.set(' ');  q.east();
+			} else if(q.isBlankX()) {
+				q.labelalias.put(buf.toString(), bf2.toString());
+				q.cr();
+				etat = S.INIT;
+			} else if(q.get() >= 0) {
+				bf2.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				throw new NinaException(this, "invalidlabeldef");
+			}
+			break;
+		case PDF_1:
+			if(q.get() == Quadro.EQ_TO_LEFT || q.get() == '\r') {
+				q.set(' ');  q.east();
+			} else if(q.get() == ' ') {
+				bf2 = new StringBuffer();
+				q.east();
+				etat = S.PDF_2;
+			} else if(q.get() >= 0) {
+				buf.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				throw new NinaException(this, "invalidlabeldef");
+			}
+			break;
+		case PDF_2:
+			if(q.get() == Quadro.EQ_TO_LEFT || q.get() == '\r') {
+				q.set(' ');  q.east();
+			} else if(q.get() == ' ') {
+				q.east();
+			} else if(q.get() >= 0) {
+				etat = S.PDF_3;
+			} else {
+				throw new NinaException(this, "invalidlabeldef");
+			}
+			break;
+		case PDF_3:
+			if(q.get() == Quadro.EQ_TO_LEFT || q.get() == '\r') {
+				q.set(' ');  q.east();
+			} else if(q.isBlankX()) {
+				q.defines.put(buf.toString(), bf2.toString());
+				q.cr();
+				etat = S.INIT;
+			} else if(q.get() >= 0) {
+				bf2.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				throw new NinaException(this, "invalidlabeldef");
+			}
+			break;
+		case REQ1:
+			if(q.get() == Quadro.EQ_TO_LEFT) {
+				q.set(' ');  q.east();
+			} else if(q.get() == ' ') {
+				bf2 = new StringBuffer();
+				q.east();
+				etat = S.REQ2;
+			} else if(q.get() >= 0) {
+				buf.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				q.requires.add(new String[] { buf.toString(), null });
+				etat = S.COMNT;
+			}
+			break;
+		case REQ2:
+			if(q.get() == Quadro.EQ_TO_LEFT) {
+				q.set(' ');  q.east();
+			} else if(q.get() >= 0) {
+				bf2.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				q.requires.add(
+						new String[] { buf.toString(), bf2.toString() });
+				etat = S.COMNT;
+			}
+			break;
+		case VTYP1:
+			if(q.get() == Quadro.EQ_TO_LEFT) {
+				q.set(' ');  q.east();
+			} else if(q.get() >= 0) {
+				buf.appendCodePoint(q.get());
+				q.set(' ');  q.east();
+			} else {
+				q.valtype = buf.toString();
+				q.options.put("valtype", q.valtype);
 				etat = S.COMNT;
 			}
 			break;
@@ -630,7 +967,7 @@ public class NinaParser {
 			} else if(q.isBlankY()) {
 				throw new NinaException(this, "unexpectedeof");
 			} else {
-				q.east();
+				q.set(' ');  q.east();
 			}
 			break;
 		case DEF1:
@@ -722,6 +1059,12 @@ public class NinaParser {
 			}
 			break;
 		case LINIT:
+			if(action == null) {
+				t = q.getName();
+				action = new DFABuilder();
+				if(subfas.containsKey(t))  subfas.put(t, action);
+			}
+
 			if(_FRAME1.equals(q.getScratch2())) {
 				q.east();
 			} else if(_FRAME2.equals(q.getScratch2())) {
@@ -734,7 +1077,7 @@ public class NinaParser {
 					buf = new StringBuffer();
 					bf2 = new StringBuffer();
 					q.east();
-					etat = S.FI_L1;
+					etat = S.FI_L0;
 				}
 			} else if(q.get() == '\'' || q.get() == '"') {
 				quote = q.get();
@@ -762,6 +1105,20 @@ public class NinaParser {
 			if(_FRAME2.equals(q.getScratch2()))  etat = S.LINIT;
 			q.east();
 			break;
+		case FI_L0:
+			if(q.get() == '\'') {
+				etat = S.FI_L1;
+			} else if(q.get() == '_' ||
+					Character.isUpperCase(q.get())) {
+				etat = S.FI_L1;
+			} else if(q.get() == '<') {
+				etat = S.FI_L1;
+			} else if(q.isFrameBefore() || q.isArrow()) {
+				etat = S.FI_L1;
+			} else {
+				etat = S.LINIT;
+			}
+			break;
 		case FI_L1:
 			if(q.get() == '\'') {
 				q.set('*');
@@ -779,25 +1136,28 @@ public class NinaParser {
 			} else if(q.isFrameBefore() || q.isArrow()) {
 				q.west();
 				etat = S.FI_L2;
-			} else  {
+			} else {
 				throw new NinaException(this, "invalidframe");
 			}
 			break;
 		case FI_L2:
 			if(q.isFrameBefore()) {
 				q.west();
-			} else {
+			} else if(q.get() >= 0) {
 				q.east();
 				q.setLabel(buf.toString());
 				q.setLabelType(bf2.toString());
 				buf = new StringBuffer();
 				bf2 = new StringBuffer();
 				etat = S.FI_E;
+			} else {
+				throw new NinaException(this, "invalidframe");
 			}
 			break;
 		case FI_L3:
 			if(q.get() == '\'') {
 				q.set('*');
+				q.east();
 				etat = S.FI_L1;
 			} else if(q.get() == '<') {
 				bf2 = new StringBuffer();
@@ -850,6 +1210,14 @@ public class NinaParser {
 			}
 			break;
 		case FI_S:
+			if(q.isFrameBefore() || q.isFrame() ||
+					q.isArrow() || q.isEntry() || q.isDone()) {
+				etat = S.FI_S2;
+			} else {
+				throw new NinaException(this, "invalidframe");
+			}
+			break;
+		case FI_S2:
 			if(q.isFrameBefore() || q.isFrame() ||
 					q.isArrow() || q.isEntry() || q.isDone()) {
 				q.transformFrame();
@@ -861,6 +1229,16 @@ public class NinaParser {
 			}
 			break;
 		case FI_W:
+			q.west();
+			if(q.isFrameBefore() || q.isFrame() ||
+					q.isArrow() || q.isEntry() || q.isDone()) {
+				q.east();
+				etat = S.FI_W2;
+			} else {
+				throw new NinaException(this, "invalidframe");
+			}
+			break;
+		case FI_W2:
 			if(q.isFrameBefore() || q.isFrame() ||
 					q.isArrow() || q.isEntry() || q.isDone()) {
 				q.transformFrame();
@@ -872,15 +1250,16 @@ public class NinaParser {
 			}
 			break;
 		case FI_N:
-			if(q.isFrameBefore() || q.isFrame() ||
+			if(q.isFrameBefore() ||
 					q.isArrow() || q.isEntry() || q.isDone()) {
 				q.transformFrame();
 				q.setScratch2(_FRAME2);
 				q.north();
-			} else {
-				q.south();
+			} else if(q.isFrame()) {
 				q.setScratch2(_FRAME1);
 				etat = S.LINIT;
+			} else {
+				throw new NinaException(this, "invalidframe");
 			}
 			break;
 		case LQUOT:
@@ -936,7 +1315,10 @@ public class NinaParser {
 			break;
 		case LPRI2:
 			if(q.isBlankY()) {
-				throw new NinaException(this, "unexpectedeof");
+				// an empty language
+//				throw new NinaException(this, "unexpectedeof");
+				infoput.pwarn("emptylanguage");
+				return action.empty(q, q.getName());
 			} else if(q.isBlankX()) {
 				q.cr();
 			} else if(STARTSYM.equals(q.getLabel())) {
@@ -1295,6 +1677,15 @@ public class NinaParser {
 				action.setEdgeEnd(q);
 				q.set(Quadro.S2);
 				q.north();
+			} else if(q.get() == '*' || q.get() == '@') {
+				if(q.getScratch() != null) {
+					throw new NinaException("notjoin");
+				} else {
+					action.link(q, null);
+					action.setAccept(q.get() == '@');
+					q.setScratch(Integer.valueOf(q.get()));
+					etat = S.ONS_0;
+				}
 			} else if(q.isLetter() || q.isDigit()) {
 				q.setEdge(Integer.valueOf((char)q.get()));
 				q.set(Quadro.S2);
@@ -1369,10 +1760,10 @@ public class NinaParser {
 				buf = new StringBuffer();
 				etat = S.WP_E1;
 			} else if(q.get() == '$') {
-				action.setEdgeEnd(q);
 				q.set(Quadro.W2);
 				q.east();
-				etat = S.AR_ES;
+				buf = new StringBuffer();
+				etat = S.AREE1;
 			} else if(q.get() == '0') {
 				buf = new StringBuffer();
 				q.set(Quadro.W2);
@@ -1382,6 +1773,15 @@ public class NinaParser {
 				buf = new StringBuffer();
 				chr = 10;
 				etat = S.AREN1;
+			} else if(q.get() == '*' || q.get() == '@') {
+				if(q.getScratch() != null) {
+					throw new NinaException("notjoin");
+				} else {
+					action.link(q, null);
+					action.setAccept(q.get() == '@');
+					q.setScratch(Integer.valueOf(q.get()));
+					etat = S.ONS_0;
+				}
 			} else if(q.isLetter() || q.isDigit()) {
 				metBord((char)q.get());
 				q.set(Quadro.W2);
@@ -1653,7 +2053,7 @@ public class NinaParser {
 				try {
 					nz = RegexpParser.parse(buf.toString());
 					metBordNFA(nz);
-				} catch (RegexParseException e) {
+				} catch(RegexParseException e) {
 					throw new NinaException(this, "invalidregex");
 				}
 				q.set(Quadro.W2);
@@ -1710,11 +2110,6 @@ public class NinaParser {
 				q.east();
 				buf = new StringBuffer();
 				etat = S.AR_EX;
-//			} else if(q.get() >= 0) {
-//				action.setMealyEdge(q.get());
-//				q.set(Quadro.W2);
-//				q.east();
-//				etat = S.AR_E;
 			} else {
 				throw new NinaException(this, "invalidmealy");
 			}
@@ -1753,6 +2148,33 @@ public class NinaParser {
 				throw new NinaException(this, "invalidedge");
 			}
 			break;
+		case AREE1:
+			if(q.get() == '{') {
+				q.set(Quadro.W2);
+				q.east();
+				etat = S.AREE2;
+			} else {
+				action.setEdgeEnd(q);
+				etat = S.AR_ES;
+			}
+			break;
+		case AREE2:
+			if(q.get() == Quadro.EQ_TO_LEFT) {
+				q.set(Quadro.W2);
+				q.east();
+			} else if(q.get() == '}') {
+				metBordLabel(buf.toString());
+				q.set(Quadro.W2);
+				q.east();
+				etat = S.AR_ES;
+			} else if(q.get() >= 0) {
+				buf.appendCodePoint(q.get());
+				q.set(Quadro.W2);
+				q.east();
+			} else {
+				throw new NinaException(this, "invalidlabel");
+			}
+			break;
 		case AR_S:
 			if(q.get() == '/') {
 				q.set(Quadro.N2);
@@ -1787,6 +2209,15 @@ public class NinaParser {
 				action.setEdgeEnd(q);
 				q.set(Quadro.N2);
 				q.south();
+			} else if(q.get() == '*' || q.get() == '@') {
+				if(q.getScratch() != null) {
+					throw new NinaException("notjoin");
+				} else {
+					action.link(q, null);
+					action.setAccept(q.get() == '@');
+					q.setScratch(Integer.valueOf(q.get()));
+					etat = S.ONS_0;
+				}
 			} else if(q.isLetter() || q.isDigit()) {
 				metBord((char)q.get());
 				q.set(Quadro.N2);
@@ -1881,6 +2312,15 @@ public class NinaParser {
 				q.set(Quadro.E2);
 				q.west();
 				etat = S.ARWN0;
+			} else if(q.get() == '*' || q.get() == '@') {
+				if(q.getScratch() != null) {
+					throw new NinaException("notjoin");
+				} else {
+					action.link(q, null);
+					action.setAccept(q.get() == '@');
+					q.setScratch(Integer.valueOf(q.get()));
+					etat = S.ONS_0;
+				}
 			} else if(q.isLetter() || q.isDigit()) {
 				chr = q.get();
 				q.set(Quadro.E2);
@@ -2194,19 +2634,9 @@ public class NinaParser {
 			if(q.isBlankX()) {
 				throw new NinaException(this, "invalidsubfa");
 			} else if(q.get() == '{') {
-				if(bf2 != null) {
-					lst.add(bf2.toString());  bf2 = null;
-				}
-
-				mp = new HashMap<String, String>();
-				for(int i = 1; i <= lst.size(); i++) {
-					mp.put(i + "", lst.get(lst.size() - i));
-				}
-				metBordRes(buf.reverse().toString(), mp);
-				lst = null;
 				q.set(Quadro.E2);
 				q.west();
-				etat = S.AR_W;
+				etat = S.ARWR5;
 			} else if(q.get() >= 0) {
 				buf.append((char)q.get());
 				q.set(Quadro.E2);
@@ -2270,14 +2700,35 @@ public class NinaParser {
 				throw new NinaException(this, "invalidsubfa");
 			}
 			break;
+		case ARWR5:
+			if(q.get() == '$') {
+				metBordLabel(buf.reverse().toString());
+				q.set(Quadro.E2);
+				q.west();
+			} else {
+				if(bf2 != null) {
+					lst.add(bf2.toString());  bf2 = null;
+				}
+
+				mp = new HashMap<String, String>();
+				for(int i = 1; i <= lst.size(); i++) {
+					mp.put(i + "", lst.get(lst.size() - i));
+				}
+				metBordRes(buf.reverse().toString(), mp);
+			}
+			lst = null;
+			etat = S.AR_W;
+			break;
 		case AR_WI:
 			if(q.isBlankX()) {
 				throw new NinaException(this, "invalidchrst");
 			} else if(q.get() == '[' && nestlv-- <= 0) {
 				q.set(Quadro.E2);  q.west();
 				etat = S.AR_Wi;
+			} else if(q.get() == ']') {
+				q.set(Quadro.E2);  q.west();
+				etat = S.ARWi2;
 			} else if(q.get() >= 0) {
-				if(q.get() == ']')  nestlv++;
 				buf.append((char)q.get());
 				q.set(Quadro.E2);  q.west();
 			} else if(q.get() == Quadro.EQ_TO_LEFT) {
@@ -2296,6 +2747,17 @@ public class NinaParser {
 				etat = S.AR_W;
 			}
 			break;
+		case ARWi2:
+			if(q.get() == '\\') {
+				buf.append('\\');  buf.append(']');
+				q.set(Quadro.E2);  q.west();
+				etat = S.AR_WI;
+			} else {
+				nestlv++;
+				buf.append(']');
+				etat = S.AR_WI;
+			}
+			break;
 		case AR_WG:
 			if(q.isBlankX()) {
 				throw new NinaException(this, "invalidregex");
@@ -2508,7 +2970,7 @@ public class NinaParser {
 			}
 			break;
 		case FRR_W:
-			if(q.isFrame() || q.isArrow() || q.isDone()) {
+			if(q.isFrame() || q.isArrow()) {
 				q.set(Quadro.DONE);
 				q.west();
 			} else if(q.isEntry()) {
@@ -2621,6 +3083,9 @@ public class NinaParser {
 			} else if(q.isArrow3()) {
 				q.south();
 				etat = S.ARRZW;
+			} else if(q.get() == '*' || q.get() == '@') {
+				q.west();
+				etat = S.ONS_W;
 			} else {
 				throw new NinaException(this, "invalidedge");
 			}
@@ -2666,6 +3131,9 @@ public class NinaParser {
 			} else if(q.isArrow3()) {
 				q.west();
 				etat = S.ARRZN;
+			} else if(q.get() == '*' || q.get() == '@') {
+				q.north();
+				etat = S.ONS_n;
 			} else {
 				throw new NinaException(this, "invalidedge");
 			}
@@ -2717,6 +3185,9 @@ public class NinaParser {
 			} else if(q.isArrow3()) {
 				q.north();
 				etat = S.ARRZE;
+			} else if(q.get() == '*' || q.get() == '@') {
+				q.east();
+				etat = S.ONS_E;
 			} else {
 				throw new NinaException(this, "invalidedge");
 			}
@@ -2762,6 +3233,9 @@ public class NinaParser {
 			} else if(q.isArrow3()) {
 				q.east();
 				etat = S.ARRZS;
+			} else if(q.get() == '*' || q.get() == '@') {
+				q.south();
+				etat = S.ONS_S;
 			} else {
 				throw new NinaException(this, "invalidedge");
 			}
@@ -3017,6 +3491,7 @@ public class NinaParser {
 				if(o != null) {
 					etat = S.TOIX2;
 				} else {
+					q.setVisiting(buf.toString());
 					etat = S.LBLE;
 				}
 			}
@@ -3381,6 +3856,82 @@ public class NinaParser {
 						this, "labelnotfound", buf.toString());
 			}
 			break;
+		case ONS_0:
+			q.north();
+			etat = S.ONS_N;
+			break;
+		case ONS_N:
+			if(q.get() == '|' || q.get() == '/' || q.get() == '\\' ||
+					q.isBranchN()) {
+				etat = S.AR_N;
+			} else {
+				q.south().east();
+				etat = S.ONS_E;
+			}
+			break;
+		case ONS_E:
+			if(q.get() == '-' || q.get() == '/' || q.get() == '\\' ||
+					q.isBranchE()) {
+				etat = S.AR_E;
+			} else {
+				q.south().west();
+				etat = S.ONS_S;
+			}
+			break;
+		case ONS_S:
+			if(q.get() == '|' || q.get() == '/' || q.get() == '\\' ||
+					q.isBranchS()) {
+				etat = S.AR_S;
+			} else {
+				q.north().west();
+				etat = S.ONS_W;
+			}
+			break;
+		case ONS_W:
+			if(q.get() == '-' || q.get() == '/' || q.get() == '\\' ||
+					q.isBranchE()) {
+				etat = S.AR_E;
+			} else {
+				q.north().east();
+				etat = S.ONS_n;
+			}
+			break;
+		case ONS_n:
+			if(q.get() == Quadro.N2 || q.isBranch()) {
+				etat = S.ARR_N;
+			} else {
+				q.south().east();
+				etat = S.ONS_e;
+			}
+			break;
+		case ONS_e:
+			if(q.get() == Quadro.E2 || q.isBranch()) {
+				etat = S.ARR_E;
+			} else {
+				q.south().west();
+				etat = S.ONS_s;
+			}
+			break;
+		case ONS_s:
+			if(q.get() == Quadro.S2 || q.isBranch()) {
+				etat = S.ARR_S;
+			} else {
+				q.north().west();
+				etat = S.ONS_w;
+			}
+			break;
+		case ONS_w:
+			if(q.get() == Quadro.W2 || q.isBranch()) {
+				etat = S.ARR_W;
+			} else {
+				throw new RuntimeException();
+			}
+			break;
+		}
+
+		counter = z == etat ? counter + 1 : 0;
+		if(counter > 100000) {
+			throw new RuntimeException(z.toString());
 		}
 		return null;
 	}
