@@ -53,12 +53,21 @@ public class Quadro implements NinaEvent {
 
 	//
 	private static final String LOPT = "targetLanguage";
+	private static final String EOG = "-- end of grammar --";
 	private static final Pattern PTN1 = Pattern.compile(
 			"--[ \t]+(.+)[ \t]+--");
 	private static final Pattern PTN2 = Pattern.compile("%.*\\{");
+	private static final Pattern PTN3 = Pattern.compile(
+			"grammar:[ \t]*--[ \t]+(.+)[ \t]+--");
+	private static final Pattern PTN4 = Pattern.compile(
+			"import:[ \t]*(.+)[ \t]+as[ \t]+(.+)[ \t]*");
+	private static final Pattern PTN5 = Pattern.compile(
+			"import-grammar:[ \t]*(.+)[ \t]+as[ \t]+(.+)[ \t]*");
+	private static final Pattern CDLM = Pattern.compile("-+");
 
 	//
 	private static boolean iswarning = false;
+	private static String tabs = "    ";
 	public static PrintStream errst = null;
 
 	//
@@ -91,7 +100,7 @@ public class Quadro implements NinaEvent {
 	Map<String, StringBuffer> buffers =
 			new HashMap<String, StringBuffer>();
 	List<String> imports = new ArrayList<String>();
-	String fragment, type;
+	String fragment, type, lexer, ltype;
 
 	//
 	private int[][] quadro;
@@ -102,17 +111,26 @@ public class Quadro implements NinaEvent {
 	private int curx = 0, cury = 0;
 	private Object edge;
 	private Map<String, Quadro> subquadro;
+	private Map<String, LRObject> lrobjects;
 	private LintCells lint;
+	private List<String[]> ids;
+	List<String[]> requires = new ArrayList<String[]>();
+	Map<String, String> labelalias = new HashMap<String, String>();
+	Map<String, String> defines = new HashMap<String, String>();
+	Map<String, Integer> consts = new HashMap<String, Integer>();
+	StringWriter license = new StringWriter();
 	String rootResource;
 	String rootPackage;
 	String name;
+	String valtype;
+	String description;
 
 	//
 	private NinaLanguageOption langopt = null;
 
 	//
 	Quadro(String n, List<int[]> q, String f, Map<String, Quadro> sub,
-			Map<String, String> opts) {
+			Map<String, LRObject> lros, Map<String, String> opts) {
 		options   = opts;
 		name      = n;
 		fragment  = f;
@@ -124,6 +142,7 @@ public class Quadro implements NinaEvent {
 		visited   = new String[q.size()][];
 		visiting  = new String[q.size()][];
 		subquadro = sub;
+		lrobjects = lros;
 		for(int i = 0; i < q.size(); i++) {
 			quadro[i]   = q.get(i);
 			scratch[i]  = new Object[q.get(i).length];
@@ -138,7 +157,7 @@ public class Quadro implements NinaEvent {
 
 	//
 	Quadro(String n, int[][] q, String f, Map<String, Quadro> sub,
-			Map<String, String> opts) {
+			Map<String, LRObject> lros, Map<String, String> opts) {
 		options   = opts;
 		name      = n;
 		fragment  = f;
@@ -150,6 +169,7 @@ public class Quadro implements NinaEvent {
 		visited   = new String[q.length][];
 		visiting  = new String[q.length][];
 		subquadro = sub;
+		lrobjects = lros;
 		for(int i = 0; i < q.length; i++) {
 			quadro[i]   = new int[q[i].length];
 			scratch[i]  = new Object[q[i].length];
@@ -167,8 +187,9 @@ public class Quadro implements NinaEvent {
 
 	//
 	Quadro(String n, List<int[]> q, String f,
+			Map<String, LRObject> lros,
 			Map<String, String> opts) {
-		this(n, q, f, null, opts);
+		this(n, q, f, null, lros, opts);
 	}
 
 	/**
@@ -179,6 +200,18 @@ public class Quadro implements NinaEvent {
 		this.options = options;
 	}
 
+	/**
+	 * 
+	 * @param x
+	 */
+	public static void setTab(int x) {
+		StringBuffer b = new StringBuffer();
+
+		if(x <= 0)  return;
+		for(int k = 0; k < x; k++)  b.append(' ');
+		tabs = b.toString();
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.NinaEvent#getRootResource()
 	 */
@@ -644,14 +677,33 @@ public class Quadro implements NinaEvent {
 					Collections.<String, Quadro>emptyMap();
 	}
 
+	/**
+	 * 
+	 * @param name
+	 * @return
+	 */
+	public LRObject getLRObject(String name) {
+		return lrobjects != null ? lrobjects.get(name) : null;
+	}
+
+	/**
+	 * 
+	 * @return
+	 */
+	public Map<String, LRObject> getLRObjects() {
+		return lrobjects != null ?
+				Collections.unmodifiableMap(lrobjects) :
+					Collections.<String, LRObject>emptyMap();
+	}
+
 	/**
 	 * 
 	 * @param q
 	 * @return
 	 */
 	public Quadro prototype(Quadro q) {
-		return new Quadro(name, quadro, fragment,
-				q.getSubQuadros(), q.options);
+		return new Quadro(name, quadro, fragment, q.getSubQuadros(),
+				lrobjects, q.options);
 	}
 
 	/**
@@ -781,45 +833,6 @@ public class Quadro implements NinaEvent {
 		return getStateByLabel(NinaParser.DEADSYM);
 	}
 
-	//
-	static int len(char c) {
-		Character.UnicodeBlock b;
-
-		b = Character.UnicodeBlock.of(c);
-		if(b == null) {
-			return 1;
-		} else if(c >= 0xff01 && c <= 0xff60) {
-			return 2;
-		} else if(c >= 0xffe0 && c <= 0xffe6) {
-			return 2;
-		} else if(b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY) ||
-				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_FORMS) ||
-				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS) ||
-				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT) ||
-				b.equals(Character.UnicodeBlock.CJK_RADICALS_SUPPLEMENT) ||
-//				b.equals(Character.UnicodeBlock.CJK_STROKES) ||
-				b.equals(Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION) ||
-				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS) ||
-				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A) ||
-				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B) ||
-				b.equals(Character.UnicodeBlock.KATAKANA) ||
-				b.equals(Character.UnicodeBlock.HIRAGANA) ||
-				b.equals(Character.UnicodeBlock.HANGUL_SYLLABLES)) {
-			return 2;
-		} else {
-			return 1;
-		}
-	}
-
-	static int len(String s) {
-		int r = 0;
-
-		for(int i = 0; i < s.length(); i++) {
-			r = r + len(s.charAt(i));
-		}
-		return r;
-	}
-
 	static int putWedgeN(int x) {
 		if(x == '+') {
 			return BRANCH - BRANCH_N;
@@ -866,14 +879,15 @@ public class Quadro implements NinaEvent {
 	 * @return
 	 * @throws IOException
 	 */
-	public static Quadro readResource(String name) throws IOException {
+	public static Quadro readResource(String name,
+			List<String> l) throws IOException {
 		InputStream in = null;
 		Reader rd;
 
 		try {
 			in = Quadro.class.getResourceAsStream(name);
 			rd = new InputStreamReader(in);
-			return read(name, rd);
+			return read(name, rd, l);
 		} finally {
 			if(in != null)  in.close();
 		}
@@ -884,30 +898,54 @@ public class Quadro implements NinaEvent {
 	 * @param s
 	 * @return
 	 */
-	public static Quadro read(String name, String s) {
+	public static Quadro read(String name, String s, List<String> l) {
 		StringReader r;
 
 		try {
 			r = new StringReader(s);
-			return read(name, r);
+			return read(name, r, l);
 		} catch (IOException e) {
 			throw new RuntimeException(e);
 		}
 	}
 
-	private static int[] toiarray(String s) {
-		int[] a;
-		int m;
+	//
+	private static Quadro putquadro(Map<String, Quadro> q,String x,
+			List<int[]> l, Map<String, String> m, int y) {
+		Quadro v;
 
-		a = new int[len(s)];
-		for(int i = 0, j = 0; j < s.length(); i += m, j++) {
-			m = len(s.charAt(j));
-			a[i] = s.charAt(j);
-			for(int k = i + 1; k < i + m; k++) {
-				a[k] = Quadro.EQ_TO_LEFT;
+		q.put(x, v = new Quadro(x, l, "", null, m));
+		v.lint.setYOffset(y);
+		iswarning = v.lint.validate(errst) | iswarning;
+		return v;
+	}
+
+	//
+	private static List<int[]> setquadro(Map<String, Quadro> q,
+			Map<String, String> m, String x, int y, List<int[]> z,
+			List<int[]> l) {
+		if(z == null) {
+			return l;
+		} else if(x != null) {
+			putquadro(q, x, l, m, y);
+		}
+		return z;
+	}
+
+	private static LRObject readlr(String fn,
+			List<String> libs) throws IOException {
+		InputStream in;
+		Reader rd = null;
+
+		try {
+			if((in = Nina.findLib(fn, libs)) == null) {
+				throw new NinaException("filenotfound", fn);
 			}
+			rd = new InputStreamReader(in);
+			return LRObject.parse(rd);
+		} finally {
+			if(rd != null)  rd.close();
 		}
-		return a;
 	}
 
 	/**
@@ -917,74 +955,99 @@ public class Quadro implements NinaEvent {
 	 * @return
 	 * @throws IOException
 	 */
-	public static Quadro read(String name,
-			Reader rd) throws IOException {
+	public static Quadro read(String name, Reader rd,
+			List<String> libs) throws IOException {
+		Map<String, LRObject> f = new HashMap<String, LRObject>();
 		Map<String, Quadro> q = new HashMap<String, Quadro>();
 		Map<String, String> m = new HashMap<String, String>();
 		List<int[]> l = new ArrayList<int[]>(), z = null;
+		List<String[]> d = new ArrayList<String[]>();
 		BufferedReader br = new BufferedReader(rd);
 		List<String> sl = new ArrayList<String>();
-		StringWriter sw = new StringWriter();
-		PrintWriter pw = new PrintWriter(sw);
-		String s, x = null;
-		boolean b = false;
+		String s, x = null, desc = "";
+		boolean b = false, b2 = true;
+		StringWriter sw = null;
+		PrintWriter pw = null;
 		int y = 0, p = 0;
+		LRObject lr;
+		String[] a;
 		Matcher h;
 		Quadro v;
-		int[] a;
 
-		for(; (s = br.readLine()) != null && !s.equals("%%"); p++) {
-			if(s.equals("")) {
+		for(; (s = br.readLine()) != null && !(sw == null && s.equals("%%")); p++) {
+			s = s.replaceAll("\t", tabs);
+			s = s.replaceAll("\u3000", "  ");
+			if(sw != null) {
+				if(!b2) {
+					if(b2 = CDLM.matcher(s).matches()) {
+						desc = sw.toString();
+						sw = null;
+					} else {
+						pw.println(s);
+					}
+				} else if(s.equals(EOG)) {
+					lr = LRObject.parse(new StringReader(sw.toString()));
+					f.put(x, lr);
+					sw = null;  x = null;
+				} else {
+					pw.println(s);
+				}
+			} else if(s.equals("")) {
+				continue;
+			} else if(CDLM.matcher(s).matches()) {
+				b2 = false;
+				sw = new StringWriter();
+				pw = new PrintWriter(sw, true);
+			} else if((h = PTN1.matcher(s)).matches()) {
+				z = setquadro(q, m, x, y, z, l);
+				y = p + 1;
+				x = h.group(1);
+				l = new ArrayList<int[]>();
+				for(String t : sl)  l.add(Nina.toiarray(t));
+			} else if((h = PTN3.matcher(s)).matches()) {
+				z = setquadro(q, m, x, y, z, l);
+				x = h.group(1);
+				sw = new StringWriter();
+				pw = new PrintWriter(sw, true);
+			} else if((h = PTN4.matcher(s)).matches()) {
+				z = setquadro(q, m, x, y, z, l);
+				a = Nina.readHeader(h.group(1), libs);
+				putquadro(q, h.group(2),
+						Nina.readImport(h.group(1), libs, a[2]), m, y);
+				if(a[0] != null)  d.add(a);
+				x = null;
+			} else if((h = PTN5.matcher(s)).matches()) {
+				z = setquadro(q, m, x, y, z, l);
+				a = Nina.readHeader(h.group(1), libs);
+				f.put(h.group(2), readlr(h.group(1), libs));
+				if(a[0] != null)  d.add(a);
+				x = null;
+			} else if(z != null && x == null) {
 				continue;
 			} else if(!b && z == null && s.charAt(0) == '#') {
 				sl.add(s);
-				a = toiarray(s);
-				l.add(a);
+				l.add(Nina.toiarray(s));
 			} else if(!b && PTN2.matcher(s).lookingAt()) {
 				b = true;
-				a = toiarray(s);
-				l.add(a);
+				l.add(Nina.toiarray(s));
 			} else if(b && s.startsWith("%}")) {
 				b = false;
-				a = toiarray(s);
-				l.add(a);
-			} else if(!(h = PTN1.matcher(s)).matches()) {
-				a = toiarray(s);
-				l.add(a);
+				l.add(Nina.toiarray(s));
 			} else {
-				if(z != null) {
-					q.put(x, v = new Quadro(x, l, "", m));
-					v.lint.setYOffset(y);
-					iswarning = v.lint.validate(errst) | iswarning;
-					y = p + 1;
-				} else {
-					z = l;
-				}
-
-				x = h.group(1);
-				l = new ArrayList<int[]>();
-				for(String t : sl) {
-					a = toiarray(t);
-					l.add(a);
-				}
+				l.add(Nina.toiarray(s));
 			}
 		}
 
-		while((s = br.readLine()) != null) {
-			pw.println(s);
-		}
-		pw.flush();
-
-		if(z != null) {
-			q.put(x, v = new Quadro(x, l, "", m));
-			v.lint.setYOffset(y);
-			iswarning = v.lint.validate(errst) | iswarning;
-		} else {
-			z = l;
-		}
-		v = new Quadro(name, z, sw.toString(), q,
+		if(sw != null)  throw new NinaException("unexpectedeof");
+		z = setquadro(q, m, x, y, z, l);
+		sw = new StringWriter();
+		pw = new PrintWriter(sw, true);
+		while((s = br.readLine()) != null)  pw.println(s);
+		v = new Quadro(name, z, sw.toString(), q, f,
 				new HashMap<String, String>());
 		v.lint.setYOffset(0);
+		v.ids = d;
+		v.description = desc;
 		iswarning = v.lint.validate(errst) | iswarning;
 		return v;
 	}
@@ -998,6 +1061,22 @@ public class Quadro implements NinaEvent {
 		return options.get(s);
 	}
 
+	/**
+	 * 
+	 * @return
+	 */
+	public String getDescription() {
+		return description;
+	}
+
+	/**
+	 * 
+	 * @return
+	 */
+	public String getLicense() {
+		return license.toString();
+	}
+
 	/**
 	 * 
 	 * @param s
@@ -1164,6 +1243,101 @@ public class Quadro implements NinaEvent {
 		return iswarning;
 	}
 
+	/**
+	 * 
+	 * @return
+	 */
+	public Map<String, Integer> getConstantMap() {
+		return Collections.unmodifiableMap(consts);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaEvent#isGrammar(java.lang.String)
+	 */
+	@Override
+	public boolean isGrammar(String t) {
+		return lrobjects.containsKey(t);
+	}
+
+	/**
+	 * 
+	 * @return
+	 */
+	public String getLexer() {
+		return lexer;
+	}
+
+	/**
+	 * 
+	 * @return
+	 */
+	public String getLexerType() {
+		return ltype;
+	}
+
+	private void checkRequire1(String[] a) {
+		VersionNumber f, g;
+
+		if(ids != null) {
+			for(String[] b : ids) {
+				if(a[0].equals(b[0])) {
+					if(a[1] == null)  return;
+					if(b[1] == null) {
+						throw new NinaException(
+								"requireversion", a[0]);
+					}
+
+					try {
+						g = new VersionNumber(b[1]);
+					} catch(NumberFormatException e) {
+						throw new NinaException("invalidversion",
+								b[1]);
+					}
+
+					try {
+						if(a[1].charAt(0) == '>') {
+							f = new VersionNumber(a[1].substring(1));
+							if(!g.isUpperThan(f)) {
+								throw new NinaException(
+										"requireversion", a[0]);
+							}
+						} else if(a[1].charAt(0) == '=') {
+							f = new VersionNumber(a[1].substring(1));
+							if(!g.isIncluded(f)) {
+								throw new NinaException(
+										"requireversion", a[0]);
+							}
+						} else if(a[1].charAt(0) == '<') {
+							f = new VersionNumber(a[1].substring(1));
+							if(!g.isLowerThan(f)) {
+								throw new NinaException(
+										"requireversion", a[0]);
+							}
+						} else {
+							f = new VersionNumber(a[1]);
+							if(!g.isUpperThan(f)) {
+								throw new NinaException(
+										"requireversion", a[0]);
+							}
+						}
+						return;
+					} catch(NumberFormatException e) {
+						throw new NinaException("invalidversion",
+								a[1]);
+					}
+				}
+			}
+		}
+		throw new NinaException("packagenotfound", a[0]);
+	}
+
+	//
+	void checkRequire() {
+		for(String[] a : requires) {
+			checkRequire1(a);
+		}
+	}
+
 	/* (non-Javadoc)
 	 * @see java.lang.Object#toString()
 	 */
